"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

from __future__ import annotations

# standard library
from enum import Enum
from typing import ClassVar

from .. import cimpl as cimpl

class ScramMechanism(Enum):
    UNKNOWN: ClassVar[int]
    SCRAM_SHA_256: ClassVar[int]
    SCRAM_SHA_512: ClassVar[int]

    def __lt__(self, other: "ScramMechanism") -> bool: ...

class ScramCredentialInfo:
    mechanism: ScramMechanism
    iterations: int

    def __init__(self, mechanism: ScramMechanism, iterations: int) -> None: ...

class UserScramCredentialsDescription:
    user: ClassVar[str]
    scram_credential_info: ClassVar[ScramCredentialInfo]

    def __init__(self, user: str, scram_credential_infos: list[ScramCredentialInfo]) -> None: ...

class UserScramCredentialAlteration:
    user: str

    def __init__(self, user: str) -> None: ...

class UserScramCredentialUpsertion(UserScramCredentialAlteration):
    user: str
    scram_credential_info: ScramCredentialInfo
    password: bytes
    salt: bytes | None

    def __init__(
        self,
        user: str,
        scram_credential_info: ScramCredentialInfo,
        password: bytes,
        salt: bytes | None = None,
    ) -> None: ...

class UserScramCredentialDeletion(UserScramCredentialAlteration):
    user: str
    mechanism: ScramMechanism

    def __init__(self, user: str, mechanism: ScramMechanism) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

from __future__ import annotations

class SerializerError(Exception):
    message: str

    def __init__(self, message: str) -> None: ...

class KeySerializerError(SerializerError): ...
class ValueSerializerError(SerializerError): ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

from __future__ import annotations

# standard library
from io import BytesIO
from typing import TYPE_CHECKING, Any, Callable

from ..serialization import Deserializer as Deserializer
from ..serialization import SerializationContext
from ..serialization import SerializationError as SerializationError
from ..serialization import Serializer as Serializer
from . import Schema as Schema
from . import topic_subject_name_strategy as topic_subject_name_strategy
from .schema_registry_client import SchemaRegistryClient

if TYPE_CHECKING:
    # pypi/conda library
    from fastavro.types import AvroMessage as AvroMessage

class _ContextStringIO(BytesIO):
    def __enter__(self) -> _ContextStringIO: ...
    def __exit__(self, *args): ...

class AvroSerializer(Serializer):
    def __init__(
        self,
        schema_registry_client: SchemaRegistryClient,
        schema_str: str | Schema,
        to_dict: Callable[[object, SerializationContext], dict[Any, Any]] | None = None,
        conf: dict | None = None,
    ) -> None: ...
    def __call__(self, obj: object, ctx: SerializationContext) -> bytes | None: ...  # type: ignore # issue: https://github.com/confluentinc/confluent-kafka-python/issues/1631

class AvroDeserializer(Deserializer):
    def __init__(
        self,
        schema_registry_client: SchemaRegistryClient,
        schema_str: str | Schema | None = None,
        from_dict: Callable[[dict[Any, Any], SerializationContext], object] | None = None,
        return_record_name: bool = False,
    ) -> None: ...
    def __call__(self, data: bytes, ctx: SerializationContext) -> "AvroMessage | dict | None": ...  # type: ignore # issue: https://github.com/confluentinc/confluent-kafka-python/issues/1631



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

from __future__ import annotations

# standard library
from enum import Enum
from typing import ClassVar

class ResourceType(Enum):
    UNKNOWN = ClassVar[int]
    ANY = ClassVar[int]
    TOPIC = ClassVar[int]
    GROUP = ClassVar[int]
    BROKER = ClassVar[int]

    def __lt__(self, other: ResourceType) -> bool: ...

class ResourcePatternType(Enum):
    UNKNOWN = ClassVar[int]
    ANY = ClassVar[int]
    MATCH = ClassVar[int]
    LITERAL = ClassVar[int]
    PREFIXED = ClassVar[int]

    def __lt__(self, other) -> bool: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

from __future__ import annotations

# standard library
from enum import Enum
from typing import ClassVar

from ..cimpl import TopicPartition

class Node:
    id: int
    id_string: str
    host: str
    port: int
    rack: str

    def __init__(self, id: int, host: str, port: int, rack: str | None = None) -> None: ...
    def __str__(self) -> str: ...

class ConsumerGroupTopicPartitions:
    group_id: str
    topic_partitions: list["TopicPartition"]

    def __init__(self, group_id: str, topic_partitions: list["TopicPartition"] | None = None) -> None: ...

class ConsumerGroupState(Enum):
    UNKNOWN: int
    UNKOWN: int
    PREPARING_REBALANCING: int
    COMPLETING_REBALANCING: int
    STABLE: int
    DEAD: int
    EMPTY: int

    def __lt__(self, other) -> bool: ...

class TopicCollection:
    topic_names: list[str]

    def __init__(self, topic_names: list[str]): ...

class TopicPartitionInfo:
    id: int
    leader: Node
    replicas: list[Node]
    isr: list[Node]

    def __init__(self, id: int, leader: Node, replicas: list[Node], isr: list[Node]): ...

class IsolationLevel(Enum):
    READ_UNCOMMITTED: ClassVar[int]
    READ_COMMITTED: ClassVar[int]

    def __lt__(self, other) -> bool: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

from __future__ import annotations

from .._model import TopicPartitionInfo
from ..cimpl import Uuid
from ._acl import AclOperation

class TopicDescription:
    name: str
    topic_id: Uuid
    is_internal: bool
    partitions: list[TopicPartitionInfo]
    authorized_operations = list[AclOperation] | None

    def __init__(
        self,
        name: str,
        topic_id: Uuid,
        is_internal: bool,
        partitions: list[TopicPartitionInfo],
        authorized_operations: list[AclOperation] | None,
    ): ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

from __future__ import annotations

# standard library
from typing import Any

from .._model import ConsumerGroupState as ConsumerGroupState
from ..error import KafkaError as KafkaError

class ClusterMetadata:
    cluster_id: str
    controller_id: int
    brokers: dict[int, "BrokerMetadata"]
    topics: dict[str, "TopicMetadata"]
    orig_broker_id: int = -1
    orig_broker_name: str | None

    def __init__(self) -> None: ...

class BrokerMetadata:
    id: int = -1
    host: str | None
    port: int = -1

    def __init__(self) -> None: ...

class TopicMetadata:
    topic: str | None = None
    partitions: dict[int, "PartitionMetadata"]
    error: "KafkaError" | None

    def __init__(self) -> None: ...

class PartitionMetadata:
    id: int
    leader: int
    replicas: list[int]
    isrs: list[int]
    error: "KafkaError | None"

    def __init__(self) -> None: ...

class GroupMember:
    id: str | None
    client_id: str | None
    client_host: str | None
    metadata: bytes | None
    assignment: bytes | None

    def __init__(self) -> None: ...

class GroupMetadata:
    broker: "BrokerMetadata | None"
    id: str | None
    error: "KafkaError | None"
    state: "ConsumerGroupState | None"
    protocol_type: Any | None
    protocol: Any
    members: list | None

    def __init__(self) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

from __future__ import annotations

# standard library
from enum import Enum
from typing import ClassVar
from typing import Type as _type

from ..cimpl import (
    ALTER_CONFIG_OP_TYPE_APPEND,
    ALTER_CONFIG_OP_TYPE_DELETE,
    ALTER_CONFIG_OP_TYPE_SET,
    ALTER_CONFIG_OP_TYPE_SUBTRACT,
    CONFIG_SOURCE_DEFAULT_CONFIG,
    CONFIG_SOURCE_DYNAMIC_BROKER_CONFIG,
    CONFIG_SOURCE_DYNAMIC_DEFAULT_BROKER_CONFIG,
    CONFIG_SOURCE_DYNAMIC_TOPIC_CONFIG,
    CONFIG_SOURCE_STATIC_BROKER_CONFIG,
    CONFIG_SOURCE_UNKNOWN_CONFIG,
)
from ..cimpl import KafkaError as KafkaError
from ._resource import ResourceType as ResourceType

class AlterConfigOpType(Enum):
    SET = ALTER_CONFIG_OP_TYPE_SET
    DELETE = ALTER_CONFIG_OP_TYPE_DELETE
    APPEND = ALTER_CONFIG_OP_TYPE_APPEND
    SUBTRACT = ALTER_CONFIG_OP_TYPE_SUBTRACT

class ConfigSource(Enum):
    UNKNOWN_CONFIG = CONFIG_SOURCE_UNKNOWN_CONFIG  #: Unknown
    DYNAMIC_TOPIC_CONFIG = CONFIG_SOURCE_DYNAMIC_TOPIC_CONFIG  #: Dynamic Topic
    DYNAMIC_BROKER_CONFIG = CONFIG_SOURCE_DYNAMIC_BROKER_CONFIG  #: Dynamic Broker
    DYNAMIC_DEFAULT_BROKER_CONFIG = CONFIG_SOURCE_DYNAMIC_DEFAULT_BROKER_CONFIG  #: Dynamic Default Broker
    STATIC_BROKER_CONFIG = CONFIG_SOURCE_STATIC_BROKER_CONFIG  #: Static Broker
    DEFAULT_CONFIG = CONFIG_SOURCE_DEFAULT_CONFIG  #: Default

class ConfigEntry:
    name: str
    value: str
    source: ConfigSource
    is_read_only: bool
    is_default: bool
    is_sensitive: bool
    is_synonym: bool
    synonyms: list[ConfigEntry]
    incremental_operation: AlterConfigOpType | None

    def __init__(
        self,
        name: str,
        value,
        source: ConfigSource | int = ConfigSource.DEFAULT_CONFIG.value,
        is_read_only: bool = False,
        is_default: bool = False,
        is_sensitive: bool = False,
        is_synonym: bool = False,
        synonyms: list[ConfigEntry] | None = None,
        incremental_operation: AlterConfigOpType | None = None,
    ) -> None: ...

class ConfigResource:
    Type = ClassVar[_type[ResourceType]]

    restype: ConfigResource
    restype_int: int
    name: str
    set_config_dict: dict[str, str]
    incremental_configs: list[ConfigEntry]
    configs: dict | None
    error: KafkaError

    def __init__(
        self,
        restype: ConfigResource,
        name: str,
        set_config: dict[str, str] | None = None,
        described_configs: dict | None = ...,
        error: "KafkaError" | None = None,
        incremental_configs: list[ConfigEntry] | None = None,
    ) -> None: ...
    def __hash__(self) -> int: ...
    def __lt__(self, other: ConfigResource) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __len__(self) -> int: ...
    def set_config(self, name: str, value, overwrite: bool = True) -> None: ...
    def add_incremental_config(self, config_entry: ConfigEntry) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

class DeletedRecords:
    low_watermark: int
    def __init__(self, low_watermark: int) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

from __future__ import annotations

# standard library
from enum import Enum, EnumMeta
from typing import Any, Literal

from .._util import ConversionUtil as ConversionUtil
from .._util import ValidationUtil as ValidationUtil
from ._resource import ResourcePatternType as ResourcePatternType
from ._resource import ResourceType as ResourceType

class AclOperation(Enum):
    UNKNOWN: int
    ANY: int
    ALL: int
    READ: int
    WRITE: int
    CREATE: int
    DELETE: int
    ALTER: int
    DESCRIBE: int
    CLUSTER_ACTION: int
    DESCRIBE_CONFIGS: int
    ALTER_CONFIGS: int
    IDEMPOTENT_WRITE: int

    def __lt__(self, other: AclOperation) -> bool: ...

class AclPermissionType(Enum):
    UNKNOWN: int
    ANY: int
    DENY: int
    ALLOW: int

    def __lt__(self, other: AclPermissionType) -> bool: ...

class AclBinding:
    restype: ResourceType
    name: str
    resource_pattern_type: ResourcePatternType
    principal: str
    host: str
    operation: AclOperation
    permission_type: AclPermissionType
    restype_int: int
    resource_pattern_type_int: int
    operation_int: int
    permission_type_int: int

    def __init__(
        self,
        restype: ResourceType,
        name: str,
        resource_pattern_type: ResourcePatternType,
        principal: str,
        host: str,
        operation: AclOperation,
        permission_type: AclPermissionType,
    ) -> None: ...
    def _convert_enums(self) -> None: ...
    def _check_forbidden_enums(self, forbidden_enums: "EnumMeta") -> None: ...
    def _not_none_args(
        self,
    ) -> Literal["restype", "name", "resource_pattern_type", "principal", "host", "operation", "permission_type"]: ...
    def _string_args(self) -> Literal["name", "principal", "host"]: ...
    def _forbidden_enums(
        self,
    ) -> dict[
        str,
        Literal[
            "RESOURCE_ANY",
            "RESOURCE_PATTERN_ANY",
            "RESOURCE_PATTERN_MATCH",
            "ACL_OPERATION_ANY",
            "ACL_PERMISSION_TYPE_ANY",
        ],
    ]: ...
    def _convert_args(self) -> None: ...
    def _to_tuple(self) -> tuple[Any]: ...
    def __repr__(self) -> str: ...
    def __hash__(self) -> int: ...
    def __lt__(self, other: "AclBinding") -> bool: ...
    def __eq__(self, other: object) -> bool: ...

class AclBindingFilter(AclBinding):
    def __init__(
        self,
        restype: ResourceType,
        name: str,
        resource_pattern_type: ResourcePatternType,
        principal: str,
        host: str,
        operation: AclOperation,
        permission_type: AclPermissionType,
    ) -> None: ...
    def _not_none_args(self) -> Literal["restype", "resource_pattern_type", "operation", "permission_type"]: ...
    def _forbidden_enums(  # type: ignore[override]
        self,
    ) -> dict[
        str,
        Literal["RESOURCE_UNKNOWN", "RESOURCE_PATTERN_UNKNOWN", "ACL_OPERATION_UNKNOWN", "ACL_PERMISSION_TYPE_UNKNOWN"],
    ]: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

from __future__ import annotations

from .._model import ConsumerGroupState as ConsumerGroupState
from .._model import Node as Node
from .._util import ConversionUtil as ConversionUtil
from ..cimpl import TopicPartition as TopicPartition
from ..error import KafkaException as KafkaException
from ._acl import AclOperation as AclOperation

class ConsumerGroupListing:
    group_id: str
    is_simple_consumer_group: bool
    state: ConsumerGroupState

    def __init__(
        self,
        group_id: str,
        is_simple_consumer_group: bool,
        state: ConsumerGroupState | None = None,
    ) -> None: ...

class ListConsumerGroupsResult:
    valid: list[ConsumerGroupListing]
    errors: list[KafkaException]

    def __init__(
        self,
        valid: list["ConsumerGroupListing"] | None = None,
        errors: list[KafkaException] | None = None,
    ) -> None: ...

class MemberAssignment:
    topic_partitions: list[TopicPartition]

    def __init__(self, topic_partitions: list[TopicPartition] | None = None) -> None: ...

class MemberDescription:
    member_id: str
    client_id: str
    host: str
    assignment: MemberAssignment
    group_instance_id: str

    def __init__(
        self,
        member_id: str,
        client_id: str,
        host: str,
        assignment: MemberAssignment,
        group_instance_id: str | None = None,
    ) -> None: ...

class ConsumerGroupDescription:
    group_id: str
    is_simple_consumer_group: bool
    members: list[MemberDescription]
    partition_assignor: str
    state: ConsumerGroupState
    coordinator: Node
    authorized_operations: list[AclOperation] | None

    def __init__(
        self,
        group_id: str,
        is_simple_consumer_group: bool,
        members: list[MemberDescription],
        partition_assignor: str,
        state: ConsumerGroupState,
        coordinator: Node,
        authorized_operations: list[AclOperation] | None = None,
    ) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

from .._model import Node
from ._acl import AclOperation

class DescribeClusterResult:
    controller: Node
    nodes: list[Node]
    cluster_id: str | None
    authorized_operations: list[AclOperation] | None
    def __init__(
        self,
        controller: Node,
        nodes: list[Node],
        cluster_id: str | None = None,
        authorized_operations: list[AclOperation] | None = None,
    ): ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

# standard library
from abc import ABC, abstractmethod

class OffsetSpec(ABC):
    @property
    @abstractmethod
    def _value(self) -> int: ...
    @classmethod
    def _fill_values(cls) -> None: ...
    @classmethod
    def earliest(cls) -> EarliestSpec: ...
    @classmethod
    def latest(cls) -> LatestSpec: ...
    @classmethod
    def max_timestamp(cls) -> MaxTimestampSpec: ...
    @classmethod
    def for_timestamp(cls, timestamp) -> TimestampSpec: ...
    def __new__(cls, index: int) -> OffsetSpec: ...
    def __lt__(self, other: OffsetSpec) -> bool: ...

class TimestampSpec(OffsetSpec):
    @property
    def _value(self) -> int: ...
    def __new__(cls, _: int) -> TimestampSpec: ...
    def __init__(self, timestamp: int) -> None: ...

class MaxTimestampSpec(OffsetSpec):
    def __new__(cls) -> MaxTimestampSpec: ...
    @property
    def _value(self) -> int: ...

class LatestSpec(OffsetSpec):
    def __new__(cls) -> LatestSpec: ...
    @property
    def _value(self) -> int: ...

class EarliestSpec(OffsetSpec):
    def __new__(cls) -> EarliestSpec: ...
    @property
    def _value(self) -> int: ...

class ListOffsetsResultInfo:
    def __init__(self, offset: int, timestamp: int, leader_epoch: int | None): ...
